/* ClockPillsGL.c : Draw a textured 3D clock with 148 lines of code
If OpenGL was too difficult for you this example will change your mind      */

/* ClockPillsGL.c : Trace une horloge texture avec 148 lignes de code
Si OpenGL etait trop complique pour toi Tu changera d'avis avec cet exemple */

/* This program is GPL - Alain Thellier - Paris - France - 2008             */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <GL/glut.h>


static GLfloat yRot = 0.0;
GLuint windowID;
unsigned char *textures[2];
GLuint texturesID[2];
int h,m,s;

#include "ClockPillsGL.h"

static void DrawObject(float *points,int *indices,int trianglesCount,int n)
{
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glTexCoordPointer(2, GL_FLOAT, 5*4,&points[0]);
	glVertexPointer(  3, GL_FLOAT, 5*4,&points[2]);
	glBindTexture(GL_TEXTURE_2D, texturesID[n]);
	glDrawElements(GL_TRIANGLES, trianglesCount*3, GL_UNSIGNED_INT, indices);

	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
}

void SetPosition(float x,float y,float z,float rx,float ry,float rz)
{
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glRotatef(rx, 1.0f, 0.0f, 0.0f);
	glRotatef(ry, 0.0f, 1.0f, 0.0f);
	glRotatef(rz, 0.0f, 0.0f, 1.0f);
	glTranslatef(x,y,z);

}


void xytext(float x,float y,unsigned char *t)
{
void *font = GLUT_BITMAP_8_BY_13;

	SetPosition(0.0,0.0,0.0,0.0,0.0,0.0);
	glColor4ub(255,255,255,255);
	glRasterPos2f(10,10);
	while (*t != 0)
		glutBitmapCharacter(font,*t++);
}

static void DisplayFunc(void)
{
unsigned char t[20];

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

	SetPosition(0.0,0.0,0.0,0.0,0.0,0.0);
	DrawObject(BodyPoints,      BodyIndices,BodyTrianglesCount,1);

	SetPosition(0.0,0.0,-0.001,0.0,0.0,360.0-(h+m/60.0)*30.0);
	DrawObject(HandsPoints,  HourHandIndices,HandTrianglesCount,0);

	SetPosition(0.0,0.0,-0.002,0.0,0.0,360.0-m*6.0);
	DrawObject(HandsPoints,MinuteHandIndices,HandTrianglesCount,0);

	SetPosition(0.0,0.0,-0.003,0.0,0.0,360.0-s*6.0);
	DrawObject(HandsPoints,SecondHandIndices,HandTrianglesCount,0);
	glutSwapBuffers();

	sprintf(t,"%d:%d:%d",h,m,s);
	xytext(0.45,0.45,t);


}

static void IdleFunc(void)
{
struct tm *newtime;
time_t mytime;

	time(&mytime);
	newtime = localtime(&mytime);
	h=newtime->tm_hour;
	m=newtime->tm_min;
	s=newtime->tm_sec;

	if(s<30)
	yRot = 3.0*s;
	else
	yRot = 360.0 - 3.0*(s-30);


	glutPostRedisplay();
}

static void ReshapeFunc(GLsizei width, GLsizei height)
{
	GLfloat h = (GLfloat) height / (GLfloat) width;

	glViewport(0, 0,  width,  height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
/*	glFrustum(-1.0, 1.0, -h, h, 0.0, 1.0); */

}

void VisibilityFunc(int vis)
{
	if (vis == GLUT_VISIBLE)
		glutIdleFunc(IdleFunc);
	else
		glutIdleFunc(NULL);
}

void CloseAll(void)
{
	    glutDestroyWindow(windowID);
		if(textures[0]!=NULL) free(textures[0]);
		if(textures[1]!=NULL) free(textures[1]);
		exit(1);
}
void KeyboardFunc(unsigned char key,int x, int y)
{
   switch (key)
	{
      case 27:
	      CloseAll();
	 default:
      	break;
    }
}

int LoadFile(unsigned char *buffer, char* name,long bufferSize)
{
	FILE *fp;
	long size;

	fp = fopen(name,"rb");
	if(fp == NULL)
	{
		printf("Cant open file !\n");
		return 0;
	}

	size = fread(buffer,bufferSize,1,fp);
	if(size ==0 )
	{
		printf("Cant read file !\n");
		return 0;
	}
	fclose(fp);
	return 1;
}

void  LoadTexture(char* name,int size,int bits,int n)
{
unsigned long textureSize = size*size*bits/8;
GLenum format;

	if (bits==32)
		format=GL_RGBA; else format=GL_RGB;
	textures[n] = (unsigned char *) malloc(textureSize);
	if(textures[n]==NULL)
	      CloseAll();

	LoadFile(textures[n],name,textureSize);
	glGenTextures(1,&texturesID[n]);
	glBindTexture(GL_TEXTURE_2D,texturesID[n]);
	glTexImage2D(GL_TEXTURE_2D,0,bits/8,size,size,0,format,GL_UNSIGNED_BYTE, textures[n]);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glEnable(GL_TEXTURE_2D);
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE );
	glutInitWindowSize(240,240);
	windowID=glutCreateWindow("ClockPillsGL - Thellier 2008 (Esc to exit)");

	LoadTexture("Texture1_128X128X32.RAW",128,32,0);
	LoadTexture("Texture2_256X256X24.RAW",256,24,1);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glClearColor(0.1f, 0.1f, 0.5f, 1.0f);
	glEnable(GL_DEPTH_TEST);
	glFrontFace(GL_CCW);
	glEnable(GL_CULL_FACE);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);

	glutDisplayFunc(DisplayFunc);
	glutReshapeFunc(ReshapeFunc);
	glutVisibilityFunc(VisibilityFunc);
	glutKeyboardFunc(KeyboardFunc);

	glutMainLoop();
	return 0;
}
