/* CoW3D.c : Draw a textured 3D object with 400 lines of code
Compiled with GCC. Use Warp3D library (needed)
Author: Alain Thellier - Paris - 2010
If Warp3D was too difficult for you this example will change your mind */

/* CoW3D.c : Trace un objet 3D texturee en 400 lignes de code
Compile avec GCC. Bibliotheque Warp3D necessaire
Auteur: Alain Thellier - Paris - 2010
Si Warp3D etait trop complique pour toi Tu changera d'avis avec cet exemple */

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <strings.h>
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <proto/Warp3D.h>
/*==================================================================*/
#include "Cow3D.h"				/* 3d object*/
#define LARGE 640					/* window size */
#define HIGH  480
#define NLOOP(nbre) for(n=0;n<nbre;n++)
#define SPEED 5					/* object speed */
#define FRAMESCOUNT (360*2/SPEED)		/* frames to be counted to estimate fps */
#define MAXPRIM (3*1000)			/* Wazp3D cant draw bigger primitives  so split the object in several drawing*/
/*==================================================================*/
struct point3D{
	float x,y,z,u,v,w;
	UBYTE RGBA[4];
};
/*==================================================================*/
struct object3D{
	struct point3D  *P;		/* original points */
	struct point3D  *P2;		/* points rotated/transformed to screen */
	ULONG Pnb;				/* points counts(french "points nombre" */
	ULONG *indices;			/* indices that define the triangles */
	ULONG Inb;				/* indices counts(french "indices nombre" */
};
/*==================================================================*/
UBYTE  progname[]={"CoW3D"};
struct Screen  *screen		=NULL;
struct Window  *window		=NULL;
struct BitMap  *bm		=NULL;
struct Library *Warp3DBase	=NULL;
W3D_Context *context		=NULL;
W3D_Texture *tex1			=NULL;
UBYTE *picture1			=NULL;
W3D_Scissor scissor = {0,0,LARGE,HIGH};
struct RastPort bufferrastport;
ULONG result,ModeID,flags,ScreenBits;
BOOL  closed=FALSE;
BOOL usetri=FALSE;
BOOL showfps=TRUE;
BOOL zbuffer=TRUE;
BOOL optimroty=FALSE;
float ViewMatrix[16];
struct object3D Obj;
float RotY=0.0;
ULONG FramesCounted=0;
ULONG FPS=0;
UBYTE FpsText[200];
/*==================================================================*/
void DrawWarp3D(struct point3D  *P,ULONG *indices,ULONG Inb,ULONG primitive,W3D_Texture *tex)
{
void *VertexPointer;
void *TexCoordPointer;
void *ColorPointer;
UWORD stride=sizeof(struct point3D);
UWORD off_v,off_w;

	W3D_WaitIdle(context);		/* dont modify points or pointers during drawing */
/* set arrays pointers */
	VertexPointer=	(void *)&(P->x);
	TexCoordPointer=(void *)&(P->u);
	ColorPointer=	(void *)&(P->RGBA);
	off_v=(UWORD)( (ULONG)&(P->v) - (ULONG)&(P->u));
	off_w=(UWORD)( (ULONG)&(P->w) - (ULONG)&(P->u));
	result=W3D_VertexPointer(context,VertexPointer,stride,W3D_VERTEX_F_F_F, 0);
	result=W3D_TexCoordPointer(context,TexCoordPointer,stride,0, off_v, off_w,W3D_TEXCOORD_NORMALIZED);
	result=W3D_ColorPointer(context,ColorPointer,stride,W3D_COLOR_UBYTE ,W3D_CMODE_RGBA,0);
/* lock hardware then draw */
	result=W3D_LockHardware(context);
	if(result==W3D_SUCCESS)
		{
		W3D_BindTexture(context,0,tex);
		W3D_DrawElements(context,primitive,W3D_INDEX_ULONG,Inb,indices);	/* draw with warp3d */
		W3D_Flush(context);
		W3D_WaitIdle(context);
		W3D_UnLockHardware(context);
		}
}

/*==================================================================*/
void TriDrawWarp3D(struct point3D  *P,ULONG *indices,ULONG Inb,W3D_Texture *tex)
{
W3D_Triangle tri;
W3D_Vertex *vn;
struct point3D  *PN;
ULONG n;

	result=W3D_LockHardware(context);
	if(result==W3D_SUCCESS)
	{
	NLOOP(Inb/3)
	{
	tri.tex=tex;
	PN=&P[indices[n*3+0]]; vn=&tri.v1;
	vn->x=P->x; vn->y=P->y; vn->z=P->z; vn->w=P->w; vn->u=P->u; vn->v=P->v;
	vn->color.r=P->RGBA[0]/255.0; vn->color.g=P->RGBA[1]/255.0;   vn->color.b=P->RGBA[2]/255.0;   vn->color.a=P->RGBA[3]/255.0;  
	PN=&P[indices[n*3+1]]; vn=&tri.v2;
	vn->x=P->x; vn->y=P->y; vn->z=P->z; vn->w=P->w; vn->u=P->u; vn->v=P->v;
	vn->color.r=P->RGBA[0]/255.0; vn->color.g=P->RGBA[1]/255.0;   vn->color.b=P->RGBA[2]/255.0;   vn->color.a=P->RGBA[3]/255.0;  
	PN=&P[indices[n*3+2]]; vn=&tri.v3;
	vn->x=P->x; vn->y=P->y; vn->z=P->z; vn->w=P->w; vn->u=P->u; vn->v=P->v;
	vn->color.r=P->RGBA[0]/255.0; vn->color.g=P->RGBA[1]/255.0;   vn->color.b=P->RGBA[2]/255.0;   vn->color.a=P->RGBA[3]/255.0;  
	W3D_DrawTriangle(context, &tri);
	}
	W3D_UnLockHardware(context);
	}
}
/*==================================================================*/
void DrawObjectWarp3D(struct object3D *O,W3D_Texture *tex)
{
ULONG n,nb,rest;

	if(usetri)
		{TriDrawWarp3D(O->P2,O->indices,O->Inb,tex);return;}
	nb=O->Inb/MAXPRIM;
	rest=O->Inb - nb*MAXPRIM;
	NLOOP(nb)
		DrawWarp3D(O->P2,&O->indices[MAXPRIM*n],MAXPRIM,W3D_PRIMITIVE_TRIANGLES,tex);
	DrawWarp3D(O->P2,&O->indices[MAXPRIM*nb],rest,W3D_PRIMITIVE_TRIANGLES,tex);
}
/*=================================================================*/
void SetP(struct point3D  *P,float x, float y, float z,float w,float u, float v,UBYTE r,UBYTE g,UBYTE b,UBYTE a)
{
	if(P==NULL) return;
	P->x=x; P->y=y; P->z=z;	P->w=w; P->u=u; P->v=v;
	P->RGBA[0]=r;P->RGBA[1]=g;P->RGBA[2]=b;P->RGBA[3]=a;
}
/*=================================================================*/
void LoadObject(struct object3D *O)
{
struct point3D  *P;
float *V;
ULONG n,size;
float x,y,z,w,u,v;
UBYTE r,g,b,a;

	O->Pnb=pointsCount;
	O->Inb=trianglesCount*3;

	size=O->Inb*sizeof(ULONG) + O->Pnb*sizeof(struct point3D)+ O->Pnb*sizeof(struct point3D); /* define all the object datas in a single malloc */
	O->indices 	=(ULONG *) malloc(size);
	O->P 		=(struct point3D *)&O->indices[O->Inb];
	O->P2		=(struct point3D *)&O->P[O->Pnb];

	NLOOP(O->Inb)
		O->indices[n]=indices[n];
	P=O->P;
	V=points;
	r=g=b=a=255;
	NLOOP(O->Pnb)
		{
		u=V[0]; v=V[1]; x=V[2]; y=V[3]; y=-y; z=V[4]; w=1.0/z;
		SetP(P,x,y,z,w,u,v,r,g,b,a); P++; V=V+5;
		}
}
/*=================================================================*/
void SetMrot(float *M,float R ,UBYTE a,UBYTE b,UBYTE c,UBYTE d)
{	/* define a rotation matrix */
float Pi =3.1416;
WORD n;
	if(M==NULL) return;
	M[0]=M[5]=M[10]=M[15]= 1.0;  	M[1]=M[2]=M[3]=M[4]=M[6]=M[7]=M[8]=M[9]=M[11]=M[12]=M[13]=M[14]=0.0;
	if(R==0.0) return;
	if(R>=360.0)
		{n=R/360.0;  R=R-n*360.0;}
	if(R<0.0)
		{n=1+R/360.0;R=R-n*360.0;}
	R= R / 180.0 * Pi;
	M[a]  =  (float)cos(R);
	M[b]  =  (float)cos(R);
	M[c]  =  (float)sin(R);
	M[d]  = -(float)sin(R);
}
/*=================================================================*/
void SetMrx(float *M,float x)		/* define X rotation matrix */
	{ SetMrot(M,x,5,10,6,9); }
/*=================================================================*/
void SetMry(float *M,float y)		/* define Y rotation matrix */
	{ SetMrot(M,y,10,0,8,2); }
/*=================================================================*/
void SetMrz(float *M,float z)		/* define Z rotation matrix */
	{ SetMrot(M,z,0,5,1,4); }
/*=================================================================*/
void TransformP(register float *M,struct point3D  *P,LONG Pnb)
{						/* transform points with a given matrix */
register float x;
register float y;
register float z;
register LONG n;

NLOOP(Pnb)
	{
	x=P->x;y=P->y;z=P->z;
	P->x= M[0]*x + M[4]*y+ M[8] *z+ M[12];
	P->y= M[1]*x + M[5]*y+ M[9] *z+ M[13];
	P->z= M[2]*x + M[6]*y+ M[10]*z+ M[14];
	P++;
	}
}
/*=================================================================*/
void YrotateP(register float *M,struct point3D  *P,LONG Pnb)
{						/* y rotate points with a given matrix : optimized TransformP() */
register float x;
register float z;
register float m0 =M[0];
register float m8 =M[8];
register float m2 =M[2];
register float m10=M[10];
register LONG n;

NLOOP(Pnb)
	{
	x=P->x;z=P->z;
	P->x= m0*x +  m8 *z;
	P->z= m2*x +  m10*z;
	P++;
	}
}
/*=================================================================*/
void ProjectP(struct point3D  *P,LONG Pnb)
{						/* project points to screen-coordinates */
register float x;
register float y;
register float z;
register float sizex=LARGE/2.0;
register float sizey=HIGH /2.0;
register float sizez=0.8  /2.0;
register LONG n;

NLOOP(Pnb)
	{
	x=P->x;y=P->y;z=P->z;
	P->x= x*sizex+sizex;
	P->y= y*sizey+sizey;
	P->z= z*sizez+sizez;
	P->w= 1.0/P->z;
	P++;
	}
}
/*=================================================================*/
BOOL LoadFile(UBYTE *buffer, UBYTE *name,ULONG bufferSize)
{						/* load a file in memory */
FILE *fp;
ULONG size=0;

	fp = fopen(name,"rb");
	if(fp == NULL)
		{printf("Cant open file ! <%s>\n",name); return(FALSE); }
	size = fread(buffer,bufferSize,1,fp);
	if(size ==0 )
		{printf("Cant read file ! <%s>\n",name); return(FALSE); }
	fclose(fp);
	return(TRUE);
}
/*=================================================================*/
BOOL LoadTextureWarp3D(UBYTE* name,UWORD size,UWORD bits)
{						/* load a RAW rgb bitmap in memory. Make it a Warp3D texture */
ULONG textureSize = size*size*bits/8;
ULONG mode;

	picture1 = (UBYTE *) malloc(textureSize);
	if(picture1==NULL)
		return(FALSE);
	if(LoadFile(picture1,name,textureSize)==FALSE)
		return(FALSE);
	if (bits==32) mode=W3D_A8R8G8B8; else mode=W3D_R8G8B8;
	tex1=W3D_AllocTexObjTags(context,&result,W3D_ATO_IMAGE,(ULONG)picture1,W3D_ATO_FORMAT,mode,W3D_ATO_WIDTH,size,W3D_ATO_HEIGHT,size,TAG_DONE);
	W3D_SetTexEnv(context,tex1,W3D_REPLACE,NULL);
	if(tex1==NULL)
		return(FALSE);
	return(TRUE);
}
/*==========================================================================*/
void	 DrawText(WORD x,WORD y,UBYTE *text)
{						/* draw a text in the window */
struct RastPort *rp;
UBYTE  BlueRGBA[]={ 60, 20,200,255};
UBYTE WhiteRGBA[]={255,255,255,255};

	if(strlen(text)==0) return;

	rp=window->RPort;
	SetDrMd(rp,JAM1);
	SetAPen(rp,255) ; 						
	SetRGB32(&screen->ViewPort,255,BlueRGBA[0]<<24,BlueRGBA[1]<<24,BlueRGBA[2]<<24);
	RectFill(rp,x-3,y-9,x+6*strlen(text)+3,y+2);		/* draw a rectangle */
	SetRGB32(&screen->ViewPort,255,WhiteRGBA[0]<<24,WhiteRGBA[1]<<24,WhiteRGBA[2]<<24);
	Move(rp,x-1,y-1);	Text(rp,text, strlen(text));		/* draw the text */
	SetAPen(rp,1) ; 				
}
/*=================================================================*/
static void DoFPS(void)
{					/* count the average "frame per second" */
static clock_t last=0;
clock_t now;

	if (++FramesCounted >= FRAMESCOUNT) 			/* ie after the object turned two times  */
	{
      now  = clock();
      FPS = (FRAMESCOUNT*CLOCKS_PER_SEC) / (now-last) ;
      last = now; FramesCounted = 0;
	sprintf(FpsText,"CoW3D: %d Fps (Object: %d triangles %d points)(zbuffer%d optimized yrot%d))",FPS,Obj.Inb/3,Obj.Pnb,zbuffer,optimroty);
	printf("%s\n",FpsText);
	}
	DrawText(3,9,FpsText);
}
/*=================================================================*/
SwitchDisplayWarp3D(void)
{					
W3D_Double Zclear=0.9999;

/* lock hardware then clear */
	W3D_FlushFrame(context);
	W3D_WaitIdle(context);
	W3D_SetDrawRegion(context,bm,0,&scissor);
	WaitBlit(); 
	BltBitMapRastPort(bufferrastport.BitMap,0,0,window->RPort,0,0,LARGE,HIGH,0xC0);	/* copy the "back buffer" to the window */

	result=W3D_LockHardware(context);
	if(result==W3D_SUCCESS)
	{
	W3D_ClearDrawRegion(context,0);
	W3D_ClearZBuffer(context, &Zclear);
	W3D_UnLockHardware(context);
	}
}
/*=================================================================*/
BOOL StartWarp3D(void)
{						/* open a window & a ratsport ("back buffer") & open warp3d & create a warp3d context  */
UWORD screenlarge,screenhigh,ok;
ULONG Flags =WFLG_ACTIVATE | WFLG_REPORTMOUSE | WFLG_RMBTRAP | WFLG_SIMPLE_REFRESH | WFLG_GIMMEZEROZERO ;
ULONG IDCMPs=IDCMP_CLOSEWINDOW | IDCMP_VANILLAKEY | IDCMP_RAWKEY | IDCMP_MOUSEMOVE | IDCMP_MOUSEBUTTONS ;

	Warp3DBase = OpenLibrary("Warp3D.library", 4L);
	if (!Warp3DBase)
		{printf("Cant open LIBS:Warp3D.library\n");	return FALSE;};

	screen 	= LockPubScreen("Workbench") ;
	screenlarge	=screen->Width; 	screenhigh	=screen->Height;

	window = OpenWindowTags(NULL,
	WA_Activate,	TRUE,
	WA_Width,		LARGE,
	WA_Height,		HIGH,
	WA_Left,		(screenlarge - LARGE)/2,
	WA_Top,		(screenhigh  -  HIGH)/2,
	WA_Title,		(ULONG)progname,
	WA_DragBar,		TRUE,
	WA_CloseGadget,	TRUE,
	WA_GimmeZeroZero,	TRUE,
	WA_Backdrop,	FALSE,
	WA_Borderless,	FALSE,
	WA_IDCMP,		IDCMPs,
	WA_Flags,		Flags,
	TAG_DONE);

	if (window==NULL)
		{printf("Cant open window\n");return FALSE;}

	InitRastPort( &bufferrastport );				/* allocate an other bitmap/rastport four double buffering */
	ScreenBits  = GetBitMapAttr( window->WScreen->RastPort.BitMap, BMA_DEPTH );
	flags = BMF_DISPLAYABLE|BMF_MINPLANES;
	bufferrastport.BitMap = AllocBitMap(LARGE,HIGH,ScreenBits, flags, window->RPort->BitMap);
	if(bufferrastport.BitMap==NULL)
		{printf("No Bitmap\n");return FALSE;}

	bm=bufferrastport.BitMap;				/* draw in this back-buffer */

	context = W3D_CreateContextTags(&result,
		W3D_CC_BITMAP,(ULONG)bm,	
		W3D_CC_YOFFSET,0,	
		W3D_CC_DOUBLEHEIGHT,FALSE,	
		W3D_CC_DRIVERTYPE,W3D_DRIVER_BEST,	
		W3D_CC_INDIRECT,FALSE,			/* seems that most Warp3d drivers dont support indirect mode */
		W3D_CC_FAST,TRUE,	
	TAG_DONE);
	if(result!=W3D_SUCCESS)
		{printf("Cant create context! (error %ld)\n",result);return(FALSE);}

	result=W3D_AllocZBuffer(context);
	if(result!=W3D_SUCCESS)
		{printf("Cant create zbuffer! (error %ld)\n",result);return(FALSE);}
	return(TRUE);
}
/*=================================================================*/
void CloseWarp3D(void)
{
	if (tex1)				W3D_FreeTexObj(context, tex1);
	if (picture1)			free(picture1);
	if (context)			W3D_DestroyContext(context);
	if (bufferrastport.BitMap)	FreeBitMap(bufferrastport.BitMap);
	if (window)				CloseWindow(window);
	if (Warp3DBase)			CloseLibrary(Warp3DBase);
	if (Obj.indices)			free(Obj.indices);
}
/*================================================================================*/
void WindowEvents(void)
{							/* manage the window  */
struct IntuiMessage *imsg;

	while( imsg = (struct IntuiMessage *)GetMsg(window->UserPort))
	{
	if (imsg == NULL) break;
	switch (imsg->Class)
		{
		case IDCMP_CLOSEWINDOW:
			closed=TRUE;				break;
		case IDCMP_VANILLAKEY:
			switch(imsg->Code)
			{
			case 'f':	showfps=!showfps;		break;
			case 'o':	optimroty=!optimroty;	break;
			case 'z':	zbuffer=!zbuffer;		break;
			case 't':	usetri=!usetri;		break;
			default:					break;
			}
			break;
		default:
			break;
		}
	if(imsg)
		{ReplyMsg((struct Message *)imsg);imsg = NULL;}
	}
}
/*=================================================================*/
void SetStatesWarp3D(void)
{
	W3D_SetState(context, W3D_BLENDING,		W3D_DISABLE);	/* non transparent */
	W3D_SetState(context, W3D_GOURAUD,		W3D_DISABLE);	/* non shaded */
	W3D_SetState(context, W3D_PERSPECTIVE,	W3D_DISABLE);	/* not needed here */

	W3D_SetState(context, W3D_TEXMAPPING,	W3D_ENABLE);	/* use textures */
	W3D_SetState(context, W3D_ZBUFFER,		W3D_ENABLE);
	W3D_SetState(context, W3D_ZBUFFERUPDATE,	W3D_ENABLE);
	W3D_SetZCompareMode(context,W3D_Z_LESS);				/* use zbuffer = remove hidden pixels */
	W3D_SetState(context, W3D_SCISSOR,		W3D_ENABLE);	/* clip to screen size */
	W3D_SetState(context, W3D_CULLFACE,		W3D_ENABLE);	/* remove hidden faces */

	W3D_SetState(context, W3D_GOURAUD,		W3D_ENABLE);	/* patch: gouraud is needed on some hardware */

	if(!zbuffer)
		W3D_SetState(context, W3D_ZBUFFER,		W3D_DISABLE);
}
/*=================================================================*/
int main(int argc, char *argv[])
{
	if(!StartWarp3D())
		goto panic;

	if(!LoadTextureWarp3D(TEXNAME,TEXSIZE,24))		/* texture datas as R8 G8 B8 */
		goto panic;
	LoadObject(&Obj);

	while(!closed)
	{
		SetStatesWarp3D();
		memcpy(Obj.P2,Obj.P,Obj.Pnb*sizeof(struct point3D));	/* copy the object's points to P2*/
		SetMry(ViewMatrix,RotY);					/* do a y rotation matrix */
		if(optimroty)
			YrotateP(ViewMatrix,Obj.P2,Obj.Pnb);		/* only y-rotate the object's points in P2*/
		else
			TransformP(ViewMatrix,Obj.P2,Obj.Pnb);		/* fully transform the object's points in P2*/
		ProjectP(Obj.P2,Obj.Pnb);					/* project points to screen */
		DrawObjectWarp3D(&Obj,tex1);					/* draw with warp3d */
		SwitchDisplayWarp3D();						/* copy to window */
		WindowEvents();							/* is window closed ? */
		if(showfps) DoFPS(); 						/* count the average "frames per second" */
		RotY=RotY+SPEED;							/* rotate */
	}
panic:
	CloseWarp3D();
	return 0;
}
/*=================================================================*/
