/* This program is GPL - Alain Thellier - Paris - France - 2008             */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <GL/glut.h>


GLuint windowID;
unsigned char *textures[2];
GLuint texturesID[2];

struct point3D{
	float x,y,z;
	float u,v,w;
	char RGBA[4];
};


/*=================================================================*/
void	SetPoint(struct point3D *P,float x,float y,float z,float u,float v,float r,float g,float b,float a)
{

	x= 2.0*x/256.0 - 1.0;
	y=-2.0*y/256.0 + 1.0;

	P->x=x;
	P->y=y;
	P->z=z;
	P->u=u;
	P->v=v;
	P->w=1/(z+0.001);

	P->RGBA[0]=255.0*r;
	P->RGBA[1]=255.0*g;
	P->RGBA[2]=255.0*b;
	P->RGBA[3]=255.0*a;
}
/*=================================================================*/
struct point3D Square1Points[4];
struct point3D Square2Points[4];
GLuint Square1Indices[4] = {0,1,2,3};
GLuint Square2Indices[4] = {0,1,2,3};
/*=================================================================*/
static void DrawObject(struct point3D *P,int *indices,int n,int t)
{
void *VertexPointer;
void *TexCoordPointer;
void *ColorPointer;
int stride=sizeof(struct point3D);


	VertexPointer=  (void *)&(P->x);
	TexCoordPointer=(void *)&(P->u);
	ColorPointer=   (void *)&(P->RGBA);

	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glTexCoordPointer(2, GL_FLOAT, stride,TexCoordPointer);
	glVertexPointer(  3, GL_FLOAT, stride,VertexPointer);
	glBindTexture(GL_TEXTURE_2D, texturesID[t]);
	glDrawElements(GL_QUADS,n, GL_UNSIGNED_INT, indices);

	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
}
/*=================================================================*/
void CloseAll(void)
{
	glutDestroyWindow(windowID);
	if(textures[0]!=NULL) free(textures[0]);
	if(textures[1]!=NULL) free(textures[1]);
	exit(1);
}
/*=================================================================*/
static void DisplayFunc(void)
{
unsigned char t[20];

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glDisable(GL_BLEND);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	DrawObject(Square1Points,Square1Indices,4,0);

	DrawObject(Square2Points,Square2Indices,4,1);


	glutSwapBuffers();
}
/*=================================================================*/
static void IdleFunc(void)
{
	glutPostRedisplay();
}
/*=================================================================*/
static void ReshapeFunc(GLsizei width, GLsizei height)
{

	GLfloat h = (GLfloat) height / (GLfloat) width;
	glViewport(0, 0,  width,  height);
	printf("%d x %d \n", width,  height);


	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
}
/*=================================================================*/
void VisibilityFunc(int vis)
{
	if (vis == GLUT_VISIBLE)
		glutIdleFunc(IdleFunc);
	else
		glutIdleFunc(NULL);
}
/*=================================================================*/
void MouseFunc(int button,int state,int x,int y)
{

}
/*=================================================================*/
void MotionFunc(int x,int y)
{

}
/*=================================================================*/
void KeyboardFunc(unsigned char key,int x, int y)
{
   switch (key)
	{
      case 27:
	      CloseAll();
	 default:
      	break;
    }
}
/*=================================================================*/
int LoadFile(unsigned char *buffer, char* name,long bufferSize)
{
	FILE *fp;
	long size;

	fp = fopen(name,"rb");
	if(fp == NULL)
	{
		printf("Cant open file !\n");
		return 0;
	}

	size = fread(buffer,bufferSize,1,fp);
	if(size ==0 )
	{
		printf("Cant read file !\n");
		return 0;
	}
	fclose(fp);
	return 1;
}
/*=================================================================*/
void  LoadTexture(char* name,int size,int bits,int t)
{
unsigned long textureSize = size*size*bits/8;
GLenum format;

	if (bits==32)
		format=GL_RGBA; else format=GL_RGB;
	textures[t] = (unsigned char *) malloc(textureSize);
	if(textures[t]==NULL)
	      CloseAll();

	LoadFile(textures[t],name,textureSize);
	glGenTextures(1,&texturesID[t]);
	glBindTexture(GL_TEXTURE_2D,texturesID[t]);
	glTexImage2D(GL_TEXTURE_2D,0,bits/8,size,size,0,format,GL_UNSIGNED_BYTE, textures[t]);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glEnable(GL_TEXTURE_2D);
}
/*=================================================================*/
int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA |  GLUT_DOUBLE );
	glutInitWindowSize(256,256);
	windowID=glutCreateWindow("Tester3D - Thellier 2008 (Esc to exit)");

	LoadTexture("Texture2_256X256X24.RAW",256,24,0);
	LoadTexture("Texture1_64X64X24.RAW",64,24,1);

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glClearColor(0.1f, 0.1f, 0.5f, 1.0f);
	glDisable(GL_DEPTH_TEST);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);

	SetPoint(&Square1Points[0],0,0,0.0001,0,0,255,255,255,255);
	SetPoint(&Square1Points[1],255,0,0.0001,0.9999,0,255,255,255,255);
	SetPoint(&Square1Points[2],255,255,0.0001,0.9999,0.9999,255,255,255,255);
	SetPoint(&Square1Points[3],0,255,0.0001,0,0.9999,255,255,255,255);

	SetPoint(&Square2Points[0],100,100,0.0001,0,0,255,255,255,255);
	SetPoint(&Square2Points[1],163,100,0.0001,0.9999,0,255,255,255,255);
	SetPoint(&Square2Points[2],163,163,0.0001,0.9999,0.9999,255,255,255,255);
	SetPoint(&Square2Points[3],100,163,0.0001,0,0.9999,255,255,255,255);

	glutDisplayFunc(DisplayFunc);
	glutReshapeFunc(ReshapeFunc);
	glutVisibilityFunc(VisibilityFunc);
	glutKeyboardFunc(KeyboardFunc);
	glutMouseFunc(MouseFunc);
	glutMotionFunc(MotionFunc);

	glutMainLoop();
	return 0;
}
