
W3D_Vertex verts2[4];
W3D_Triangles tris2;

void draw_16dithering(W3D_Context *, W3D_Float);

W3D_Texture *tex2 ;
char *_2st=NULL;
ULONG error;



void FloydSteinbergTo16bits(UBYTE *pt24, UWORD *pt16,UWORD large,UWORD high,ULONG format)
{
/* This function come from Alain Thellier - Paris - France - 2008 */
#define XLOOP(nbre) for(x=0;x<nbre;x++)
#define YLOOP(nbre) for(y=0;y<nbre;y++)

ULONG x,y,n;
float error[3]; 		/* the famous error */
float rounding[3]; 	/* Help to choose the nearer value*/
UWORD r,g,b,a;		/* value as 5/6bits or 1bits (alpha) */
float *A;			/* neighbors pixels */
float *B;
float *C;
float *D;
float *E;
float Af,Bf,Cf,Df,Ef;	/* Floyd Steinberg coefs to apply to error for each "neighbors" */
float *ptf;			/* float array to store errors */
UBYTE *RGB;			/* Current 24bits pixel (=in)*/
float *RGBf;		/* Current pixel value including previous errors*/
UWORD *RGB16;		/* Current 16bits pixel (=out)*/


/*	Floyd Steinberg algo error diffusion :
 *    +------+------+------+
 *    | E    |curr. | A    |
 *    | 0/16 |Pixel | 7/16 |
 *    | error|RGBf  | error|
 *    +------+------+------+
 *    | B    | C    | D    |
 *    | 3/16 | 5/16 | 1/16 |
 *    | error| error| error|
 *    +------+------+------+
*/

/*allocate a float buffer to store "errors" */
	ptf= (float *)malloc(high*large*3*sizeof(float));


/* set rounding value */
	if(format==W3D_A1R5G5B5 )
	{
	rounding[0]=8.0/2.0;
	rounding[1]=8.0/2.0;
	rounding[2]=8.0/2.0;
	}

	if(format==W3D_R5G6B5 )
	{
	rounding[0]=8.0/2.0;
	rounding[1]=4.0/2.0;
	rounding[2]=8.0/2.0;
	}


/* clean it */
	XLOOP(high*large*3)
		ptf[x]=0.0;

/* process all texture */

	YLOOP(high)
	XLOOP(large)
	{
/* Get current pixel a serpentine way: left to right then right to left*/
	if(y&1)
		n=y*large+((large-1)-x);
	else	
		n=y*large+x;

/* Get current pixels */
	RGBf = &ptf[n*3];
	RGB  =&pt24[n*3];
	RGB16=&pt16[n*1];

/* add total of previous errors to the current RGB value and round it to better value*/
	RGBf[0]=RGBf[0]+rounding[0]+RGB[0];
	RGBf[1]=RGBf[1]+rounding[1]+RGB[1]; 
	RGBf[2]=RGBf[2]+rounding[2]+RGB[2];

/* current color value */
	r=RGBf[0];	
	g=RGBf[1];
	b=RGBf[2];

/* manage current color value overflow*/
	if(RGBf[0]>255.0)	r=255;	
	if(RGBf[1]>255.0)	g=255;	
	if(RGBf[2]>255.0)	b=255;	
	if(RGBf[0]<0.0)	r=0;	
	if(RGBf[1]<0.0)	g=0;	
	if(RGBf[2]<0.0)	b=0;

/* error part = what wont be stored in the 16 bit pixel*/
	if(format==W3D_A1R5G5B5 )
	{
	error[0]=RGBf[0]-(r&(255-7));
	error[1]=RGBf[1]-(g&(255-7));
	error[2]=RGBf[2]-(b&(255-7));
	}

	if(format==W3D_R5G6B5 )
	{
	error[0]=RGBf[0]-(r&(255-7));
	error[1]=RGBf[1]-(g&(255-3));
	error[2]=RGBf[2]-(b&(255-7));
	}

	error[0]= error[0] - rounding[0];
	error[1]= error[1] - rounding[1];
	error[2]= error[2] - rounding[2];

	if(format==W3D_A1R5G5B5)
	{
/* used part as 1 5 5 5 bits */
 	a=1;
	r=((r&255)>>3);
	g=((g&255)>>3);
	b=((b&255)>>3);
/* store to 16bits */
	 RGB16[0] =(a<<15) + (r<<10) + (g<<5) + (b<<0);
	}

	if(format==W3D_R5G6B5 )
	{
/* used part as 5 6 5 bits */
	r=((r&255)>>3);
	g=((g&255)>>2);
	b=((b&255)>>3);
/* store to 16bits */
	RGB16[0] = (r<<11) + (g<<5) + (b<<0);
	}

/* obtain the five currents "neighbors"  */
	A=RGBf+3;
	E=RGBf-3;
	B=RGBf+3*large-3;
	C=RGBf+3*large+0;
	D=RGBf+3*large+3;

/* keep "neighbors" inside the texture */
	if(n==(large-1))
		{
		A=A-6;	/* avoid  to set the "neighbor" on RGBf */
		D=D-3;
		}
	if(n==0)
		{
		E=E+6;	/* avoid  to set the "neighbor" on RGBf */
		B=B+3;
		}
	if(y==(high-1))
		{
		B=B-3*large;
		D=D-3*large;
		C=B;		/* avoid  to set the "neighbor" on RGBf */
		}

/* Floyd Steinberg coefs to apply to error for each "neighbors" */
	if(y&1)		/* change coefs for odd/even lines */
	{
	Af=0.0/16.0;
	Bf=1.0/16.0;
	Cf=5.0/16.0;
	Df=3.0/16.0;
	Ef=7.0/16.0;
	}	
	else
	{
	Af=7.0/16.0;
	Bf=3.0/16.0;
	Cf=5.0/16.0;
	Df=1.0/16.0;
	Ef=0.0/16.0;	
	}


/*	add error to "neighbors"  */

	A[0]=A[0]+error[0]*Af;
	A[1]=A[1]+error[1]*Af;
	A[2]=A[2]+error[2]*Af;

	B[0]=B[0]+error[0]*Bf;
	B[1]=B[1]+error[1]*Bf;
	B[2]=B[2]+error[2]*Bf;

	C[0]=C[0]+error[0]*Cf;
	C[1]=C[1]+error[1]*Cf;
	C[2]=C[2]+error[2]*Cf;

	D[0]=D[0]+error[0]*Df;
	D[1]=D[1]+error[1]*Df;
	D[2]=D[2]+error[2]*Df;

	E[0]=E[0]+error[0]*Ef;
	E[1]=E[1]+error[1]*Ef;
	E[2]=E[2]+error[2]*Ef;


	}	/* end of loop */

free(ptf);
}

void VerySimpleDitherToA1R5G5B5(UBYTE *RGB, UWORD *RGB16,ULONG size)

{
int n;
#define NLOOP(nbre) for(n=0;n<nbre;n++)
UWORD r,g,b; 		/* total value including error */
UWORD r5,g5,b5,a1;	/* value as 5bits or 1bits (alpha) */

	r=g=b=0;
	NLOOP(size)
	{

		 r=r+RGB[0];
		 g=g+RGB[1];
		 b=b+RGB[2];
		 a1=1;
		r5=((r&255)>>3);
		g5=((g&255)>>3);
		b5=((b&255)>>3);
		if(r>=255) r5=255>>3;
		if(g>=255) g5=255>>3;
		if(b>=255) b5=255>>3;

		 RGB16[0] =(a1<<15) + (r5<<10) + (g5<<5) + (b5<<0);
		 r=r-(r5<<3);
		 g=g-(g5<<3);
		 b=b-(b5<<3);

		 RGB+=3;
		 RGB16+=1;
	}
  

}



void draw_16dithering(W3D_Context *context, W3D_Float a)   
{

   tex2 = W3D_AllocTexObjTags(context, &error,
		W3D_ATO_IMAGE,	  _2st,		  // The image data
		W3D_ATO_FORMAT,	 W3D_A1R5G5B5,	
		W3D_ATO_WIDTH,	  256,			 
		W3D_ATO_HEIGHT,	 256,			 
	TAG_DONE);


	if (!tex2 || error != W3D_SUCCESS) {
		printf("Error generating texture: ");
		switch(error) {
		case W3D_ILLEGALINPUT:
			printf("Illegal input\n");
			break;
		case W3D_NOMEMORY:
			printf("Out of memory\n");
			break;
		case W3D_UNSUPPORTEDTEXSIZE:
			printf("Unsupported texture size\n");
			break;
		case W3D_NOPALETTE:
			printf("Chunky texture without palette specified\n");
			break;
		case W3D_UNSUPPORTEDTEXFMT:
			printf("Texture format not supported\n");
			break;
		default:
			printf("ahem... An error has occured\n");
		}
		 }


	verts2[0].x=256;		  verts2[0].y= 0;			verts2[0].u=0;	verts2[0].v =0;	verts2[0].color.a = 1.0;
	verts2[0].color.r = a;	verts2[0].color.g = a;	   verts2[0].color.b = a;
	verts2[0].z=0.0;		  verts2[0].w=1.0-verts2[0].z;

	verts2[1].x=512;		  verts2[1].y= 0;			verts2[1].u=256;  verts2[1].v =0;	verts2[1].color.a = 1.0;
	verts2[1].color.r = a;	verts2[1].color.g = a;	   verts2[1].color.b = a;
	verts2[1].z=0.0;		  verts2[1].w=1.0-verts2[1].z;


	verts2[2].x=256;		  verts2[2].y= 256;		  verts2[2].u=0;	verts2[2].v = 256;  verts2[2].color.a = 1.0;
	verts2[2].color.r = a;	verts2[2].color.g = a;	   verts2[2].color.b = a;
	verts2[2].z=0.0;		  verts2[2].w=1.0-verts2[2].z;

	verts2[3].x=512;		  verts2[3].y= 256;		  verts2[3].u=256;  verts2[3].v =256;  verts2[3].color.a = 1.0;
	verts2[3].color.r = a;	verts2[3].color.g = a;	   verts2[3].color.b = a;
	verts2[3].z=0.0;		  verts2[3].w=1.0-verts2[3].z;


	tris2.tex = tex2;
	tris2.vertexcount = 4;
	tris2.v = verts2;


	W3D_LockHardware(context);

	W3D_DrawTriStrip(context,&tris2);

	W3D_UnLockHardware(context);


 }

